==> daemonshogi-0.1.3.tar.gz: daemonshogi-0.1.3/src/si/mate.c <==
/* SI SHOGI Library
   Copyright (C) 2002 Masahiko Tokita
   This file is part of the SI SHOGI Library.

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.w

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with The Daemonshogi; if not, write to the Free
   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
   02111-1307 USA.  */
/**
 * mate.c
 * $Source: /cvs/si/mate.c,v $
 * $Id: mate.c,v 1.50 2002/09/11 21:17:25 mik Exp $
 */

#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include "si.h"
#include "ui.h"

void addTREEChild(TREE *parent, TREE *child);
TREE *findPointerMe(TREE *tree, TREE *target);
void removeTREEChild(TREE *tree);

/**
 * bo ζ̤˵ͤߤ뤫Ĵ٤롣
 * ͤߤȯ tree ˼򥻥åȤ롣
 * ( tree ʲΥǡ¤ freeTREE() ȤäƳɬפ롣
 * freeTREE(tree->child) Τ褦ˤ롣 )
 * gn ϸߤɤߤο򥻥åȤ롣̾ 0 򥻥åȤǤ褤
 * ( gn  mate() ȤƵƽФȤ +1 ƥåȤ )
 * ɤߤο MATE_DEEP_MAX ˻ꤹ롣
 * @param bo оݤBOARD
 * @param gn 
 * @return CHECK_MATE ξ硢ͤߤ롣NOT_CHECK_MATE ϵͤߤդ
 *         ʤä
 */
MATESTATUS mate(BOARD *bo, TREE *tree, int gn) {
  MOVEINFO *mi1, *mi2;
  TREE *tree1, *tree2;
  extern int MATE_DEEP_MAX;
  int count;
  int i, j;
  int king_xy;
  extern GAME g_game;
  /* extern MOVEINFO g_mi[]; */
#ifdef USE_HASH
  extern long count_hash_hit;
  MATESTATUS mate_stat;
#endif /* USE_HASH */

#ifdef USE_GUI  
  /* GUIΥڥǥ󥰤줿٥Ȥ */
  if (gn < 3) {
    /* ǤλäƤʤĴ٤ */
    if ( gmGetGameStatus(&g_game) == SI_CHUDAN ||
	 gmGetGameStatus(&g_game) == SI_TORYO ||
	 gmGetChudanFlg(&g_game) == 1) {
      /* ǤޤλäΤǽλ */
      return NOT_CHECK_MATE;
    }
    processing_pending();
  }
#endif /* USE_GUI */

#ifdef USE_HASH
  if ( hsGetBOARD( bo, &mate_stat, bo->mi.count - gn ,0 ) ) {
    count_hash_hit++;
    /* ̤ϿƤ */
    if ( mate_stat == CHECK_MATE ) {
      /* ͤߤηФƤΤĴ٤ʤ褤 */
      return CHECK_MATE;
    } else if ( mate_stat == NOT_CHECK_MATE ) {
      /* Եͤ */
#ifdef DEBUG
      assert(1); /* ߤԵͤߤϿƤʤ */
#endif /* DEBUG */
      return NOT_CHECK_MATE;
    } else {
      /*  */
    }
  }
#endif /* USE_HASH */

  mi1 = newMOVEINFO();
  /* mi1 = &(g_mi[ (gn << 1) + 1 ]); */
  
  ohte(bo, mi1);
  
#ifdef USE_HASH_NO
  if ( mi1->count == 0 ) {
    /* ꤬ʤ */
    hsPutBOARD( bo, NOT_CHECK_MATE, bo->mi.count - gn, 0);
  }
#endif /* USE_HASH */

  for (i=0; i<mi1->count; i++) {
    boMove_mate( bo, &(mi1->te[i]), bo->next );
    king_xy = bo->code_xy[bo->next + 1][0];
    if (king_xy != 0 && boGetToBoardCount(king_xy, (bo->next == 0))) {
      /* ղˤʤäƤ */
      boBack_mate( bo, bo->next );
      continue;
    }

    mi2 = newMOVEINFO();
    /* mi2 = &(g_mi[(gn << 1) + 2]); */

    uke(bo, mi2, (bo->next == 0));
    
    if ( mi2->count == 0 && mi1->te[i].uti == FU ) {
      /* Ǥͤᡣ */
      freeMOVEINFO( mi2 );
      boBack_mate( bo, bo->next );
      continue;
    }
    
    tree1 = newTREE();
    tree1->te = mi1->te[i];
    addTREEChild( tree, tree1 );
    
    if ( mi2->count == 0 ) {
      /* ͤ */

      freeMOVEINFO( mi1 );
      freeMOVEINFO( mi2 );

      boBack_mate( bo, bo->next );

#ifdef USE_HASH
      hsPutBOARD( bo, CHECK_MATE, bo->mi.count - gn, 0);
#endif /* USE_HASH */
      return CHECK_MATE;
    }

    if ( MATE_DEEP_MAX <= gn ) {
      freeMOVEINFO( mi2 );
      boBack_mate( bo, bo->next );
      removeTREEChild( tree1 );
      continue;
    }

    count = 0;
    for (j=0; j<mi2->count; j++) {
      boMove_mate( bo, &(mi2->te[j]), (bo->next == 0));
      tree2 = newTREE();
      tree2->te = mi2->te[j];
      addTREEChild( tree1, tree2 );

      if ( mate( bo, tree2, gn + 1 ) == NOT_CHECK_MATE ) {
	/* ͤޤʤä */
	/* β(mi1->te[i])Ǥϵͤޤʤ */
	boBack_mate( bo, bo->next );
#ifdef USE_HASH_NO
	hsPutBOARD( bo, NOT_CHECK_MATE, bo->mi.count - gn, 0 );
#endif /* USE_HASH */
	goto NEXT;
      } else {
	count++;
      }
      boBack_mate( bo, bo->next );
    }
    if ( mi2->count == count ) {
      /* ͤ */

      freeMOVEINFO( mi1 );
      freeMOVEINFO( mi2 );

      boBack_mate( bo, bo->next );

#ifdef USE_HASH    
      hsPutBOARD( bo, CHECK_MATE, bo->mi.count - gn, 0 );
#endif /* USE_HASH */
      return CHECK_MATE;
    }

    NEXT:;

    freeMOVEINFO( mi2 );
    removeTREEChild( tree1 );
    boBack_mate( bo, bo->next );
  }
  
  freeMOVEINFO( mi1 );

  return NOT_CHECK_MATE;
}

/**
 * ſȤä;ؿ
 * bo ζ̤˵ͤߤ뤫Ĵ٤롣
 * ͤߤȯ tree ˼򥻥åȤ롣
 * ( tree ʲΥǡ¤ freeTREE() ȤäƳɬפ롣
 * freeTREE(tree->child) Τ褦ˤ롣 )
 * gn ϸߤɤߤο򥻥åȤ롣̾ 0 򥻥åȤǤ褤
 * ( gn  mate() ȤƵƽФȤ +1 ƥåȤ )
 * ɤߤο MATE_DEEP_MAX ˻ꤹ롣
 * @param bo оݤBOARD
 * @param gn 
 * @return CHECK_MATE ξ硢ͤߤ롣NOT_CHECK_MATE ϵͤߤդ
 *         ʤä
 */
MATESTATUS cmate(BOARD *bo, TREE *tree, int cn) {
  MOVEINFO *mi1, *mi2;
  TREE *tree1, *tree2;
  int count;
  int i, j;
  int king_xy;
  /* extern MOVEINFO g_mi[]; */
#ifdef USE_HASH
  extern long count_hash_hit;
  int mate_stat;
#endif /* USE_HASH */
  
  /* printf("cn = %d\n", cn); */

#ifdef USE_GUI
  /* GUIΥڥǥ󥰤줿٥Ȥ */
  if ((cn % 2) == 0) {
    processing_pending();
  }
#endif /* USE_GUI */

#ifdef USE_HASH
  if ( hsGetBOARD( bo, &mate_stat, bo->mi.count, 0 ) ) {
    count_hash_hit++;
    /* ̤ϿƤ */
    if ( mate_stat == CHECK_MATE ) {
      /* ͤߤηФƤΤĴ٤ʤ褤 */
      return CHECK_MATE;
    } else if ( mate_stat == NOT_CHECK_MATE ) {
      /* Եͤ */
#ifdef DEBUG
      assert(1); /* ߤԵͤߤϿƤʤ */
#endif /* DEBUG */
      return NOT_CHECK_MATE;
    } else {
      /*  */
    }
  }
#endif /* USE_HASH */

  mi1 = newMOVEINFO();
  /* mi1 = &(g_mi[ (gn << 1) + 1 ]); */
  
  ohte(bo, mi1);
  
#ifdef USE_HASH_NO
  if ( mi1->count == 0 ) {
    /* ꤬ʤ */
    hsPutBOARD( bo, NOT_CHECK_MATE, bo->mi.count, 0);
  }
#endif /* USE_HASH */

  for (i=0; i<mi1->count; i++) {
    boMove_mate( bo, &(mi1->te[i]), bo->next );
    king_xy = bo->code_xy[bo->next + 1][0];
    if (king_xy != 0 && boGetToBoardCount(king_xy, (bo->next == 0))) {
      /* ղˤʤäƤ */
      boBack_mate( bo, bo->next );
      continue;
    }

    mi2 = newMOVEINFO();
    /* mi2 = &(g_mi[(gn << 1) + 2]); */

    uke(bo, mi2, (bo->next == 0));
    
    if ( mi2->count == 0 && mi1->te[i].uti == FU ) {
      /* Ǥͤᡣ */
      freeMOVEINFO( mi2 );
      boBack_mate( bo, bo->next );
      continue;
    }
    
    tree1 = newTREE();
    tree1->te = mi1->te[i];
    addTREEChild( tree, tree1 );
    
    if ( mi2->count == 0 ) {
      /* ͤ */

      freeMOVEINFO( mi1 );
      freeMOVEINFO( mi2 );

      boBack_mate( bo, bo->next == 0);

#ifdef USE_HASH
      hsPutBOARD( bo, CHECK_MATE, bo->mi.count, 0);
#endif /* USE_HASH */
      return CHECK_MATE;
    }
    
    cn -= mi2->count;

    if (cn <= 0) {
      /* Եͤ */
      cn += mi2->count;
      
      freeMOVEINFO( mi2 );
      boBack_mate( bo, bo->next );
      removeTREEChild( tree1 );
      
      continue;
    }

    count = 0;
    for (j=0; j<mi2->count; j++) {
      boMove_mate( bo, &(mi2->te[j]), (bo->next == 0));
      tree2 = newTREE();
      tree2->te = mi2->te[j];
      addTREEChild( tree1, tree2 );

      if ( cmate( bo, tree2, cn) == NOT_CHECK_MATE ) {
	/* ͤޤʤä */
	/* β(mi1->te[i])Ǥϵͤޤʤ */
	boBack_mate( bo, bo->next );
#ifdef USE_HASH_NO
	hsPutBOARD( bo, NOT_CHECK_MATE, bo->mi.count - gn, 0 );
#endif /* USE_HASH */
	goto NEXT;
      } else {
	/* ͤ */
	count++;
      }
      boBack_mate( bo, bo->next );
    }
    if ( mi2->count == count ) {
      /* ͤ */

      freeMOVEINFO( mi1 );
      freeMOVEINFO( mi2 );

      boBack_mate( bo, bo->next );

#ifdef USE_HASH    
      hsPutBOARD( bo, CHECK_MATE, bo->mi.count, 0 );
#endif /* USE_HASH */
      return CHECK_MATE;
    }

    NEXT:;

    cn += mi2->count;
    
    freeMOVEINFO( mi2 );
    removeTREEChild( tree1 );
    boBack_mate( bo, bo->next );
  }
  
  freeMOVEINFO( mi1 );

  return NOT_CHECK_MATE;
}

/**
 * parent ˻Ҥ̤Υݥ child ä
 * @param parent ƤΥݥ
 * @param child ҤΥݥ
 */
void addTREEChild(TREE *parent, TREE *child) {
#ifdef DEBUG
  assert( parent != NULL );
  assert( child != NULL );
#endif /* DEBUG */

  if ( parent->child == NULL ) {
    parent->child = child;
    parent->child->parent = parent;
    parent->child->child = NULL;
    parent->child->brother = NULL;
  } else {
    TREE *tmp;
    tmp = parent->child;
    parent->child = child;
    parent->child->parent = parent;
    parent->child->child = NULL;
    parent->child->brother = tmp;
  }
}

/**
 * tree ηΥĥ꡼򤿤ɤä target ΰľηõƤ
 * ֤
 * @param tree оݤΥĥ꡼ؤΥݥ
 * @param target Υĥ꡼ؤΥݥ
 */
TREE *findPointerMe(TREE *tree, TREE *target) {
  TREE *tmp;
#ifdef DEBUG
  assert ( tree != NULL );
  assert ( target != NULL );
#endif /* DEBUG */

  tmp = tree;
  
  while ( 1 ) {
    if ( tmp->brother == target )
      break;
    tmp = tmp->brother;
#ifdef DEBUG
    assert ( tmp != NULL );
#endif /* DEBUG */
  }
  
  return tmp;
}

/**
 * Եͤꤷĥ꡼롣
 * @param tree ĥ꡼
 */
void removeTREEChild(TREE *tree) {
#ifdef DEBUG
  assert ( tree != NULL );
  assert ( tree->parent != NULL );
#endif /* DEBUG */

  if ( tree->child != NULL )
    freeTREE( tree->child );
  
  if ( tree->brother != NULL ) {
    if ( tree->parent->child == tree ) {
      tree->parent->child = tree->brother;
    } else {
      TREE *tmp;
      tmp = findPointerMe( tree->parent->child, tree );
      tmp->brother = tree->brother;
    }
  }
  
  tree->parent->child = NULL;

  free(tree);
}

==> daemonshogi-0.1.4.tar.gz: daemonshogi-0.1.4/src/si/mate.c <==
/* SI SHOGI Library
   Copyright (C) 2002 Masahiko Tokita
   This file is part of the SI SHOGI Library.

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.w

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with The Daemonshogi; if not, write to the Free
   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
   02111-1307 USA.  */
/**
 * mate.c
 * $Source: /cvsroot/daemonshogi/daemonshogi/src/si/mate.c,v $
 * $Id: mate.c,v 1.1.1.1 2005/12/09 09:03:09 tokita Exp $
 */

#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include "si.h"
#include "ui.h"

void addTREEChild(TREE *parent, TREE *child);
TREE *findPointerMe(TREE *tree, TREE *target);
void removeTREEChild(TREE *tree);

/**
 * bo ζ̤˵ͤߤ뤫Ĵ٤롣
 * ͤߤȯ tree ˼򥻥åȤ롣
 * ( tree ʲΥǡ¤ freeTREE() ȤäƳɬפ롣
 * freeTREE(tree->child) Τ褦ˤ롣 )
 * gn ϸߤɤߤο򥻥åȤ롣̾ 0 򥻥åȤǤ褤
 * ( gn  mate() ȤƵƽФȤ +1 ƥåȤ )
 * ɤߤο MATE_DEEP_MAX ˻ꤹ롣
 * @param bo оݤBOARD
 * @param gn 
 * @return CHECK_MATE ξ硢ͤߤ롣NOT_CHECK_MATE ϵͤߤդ
 *         ʤä
 */
MATESTATUS mate(BOARD *bo, TREE *tree, int gn) {
  MOVEINFO *mi1, *mi2;
  TREE *tree1, *tree2;
  extern int MATE_DEEP_MAX;
  int count;
  int i, j;
  int king_xy;
  extern GAME g_game;
  /* extern MOVEINFO g_mi[]; */
#ifdef USE_HASH
  extern long count_hash_hit;
  MATESTATUS mate_stat;
#endif /* USE_HASH */

#ifdef USE_GUI  
  /* GUIΥڥǥ󥰤줿٥Ȥ */
  if (gn < 3) {
    /* ǤλäƤʤĴ٤ */
    if ( gmGetGameStatus(&g_game) == SI_CHUDAN ||
	 gmGetGameStatus(&g_game) == SI_TORYO ||
	 gmGetChudanFlg(&g_game) == 1) {
      /* ǤޤλäΤǽλ */
      return NOT_CHECK_MATE;
    }
    processing_pending();
  }
#endif /* USE_GUI */

#ifdef USE_HASH
  if ( hsGetBOARD( bo, &mate_stat, bo->mi.count - gn ,0 ) ) {
    count_hash_hit++;
    /* ̤ϿƤ */
    if ( mate_stat == CHECK_MATE ) {
      /* ͤߤηФƤΤĴ٤ʤ褤 */
      return CHECK_MATE;
    } else if ( mate_stat == NOT_CHECK_MATE ) {
      /* Եͤ */
#ifdef DEBUG
      assert(1); /* ߤԵͤߤϿƤʤ */
#endif /* DEBUG */
      return NOT_CHECK_MATE;
    } else {
      /*  */
    }
  }
#endif /* USE_HASH */

  mi1 = newMOVEINFO();
  /* mi1 = &(g_mi[ (gn << 1) + 1 ]); */
  
  ohte(bo, mi1);
  
#ifdef USE_HASH_NO
  if ( mi1->count == 0 ) {
    /* ꤬ʤ */
    hsPutBOARD( bo, NOT_CHECK_MATE, bo->mi.count - gn, 0);
  }
#endif /* USE_HASH */

  for (i=0; i<mi1->count; i++) {
    boMove_mate( bo, &(mi1->te[i]), bo->next );
    king_xy = bo->code_xy[bo->next + 1][0];
    if (king_xy != 0 && boGetToBoardCount(king_xy, (bo->next == 0))) {
      /* ղˤʤäƤ */
      boBack_mate( bo, bo->next );
      continue;
    }

    mi2 = newMOVEINFO();
    /* mi2 = &(g_mi[(gn << 1) + 2]); */

    uke(bo, mi2, (bo->next == 0));
    
    if ( mi2->count == 0 && mi1->te[i].uti == FU ) {
      /* Ǥͤᡣ */
      freeMOVEINFO( mi2 );
      boBack_mate( bo, bo->next );
      continue;
    }
    
    tree1 = newTREE();
    tree1->te = mi1->te[i];
    addTREEChild( tree, tree1 );
    
    if ( mi2->count == 0 ) {
      /* ͤ */

      freeMOVEINFO( mi1 );
      freeMOVEINFO( mi2 );

      boBack_mate( bo, bo->next );

#ifdef USE_HASH
      hsPutBOARD( bo, CHECK_MATE, bo->mi.count - gn, 0);
#endif /* USE_HASH */
      return CHECK_MATE;
    }

    if ( MATE_DEEP_MAX <= gn ) {
      freeMOVEINFO( mi2 );
      boBack_mate( bo, bo->next );
      removeTREEChild( tree1 );
      continue;
    }

    count = 0;
    for (j=0; j<mi2->count; j++) {
      boMove_mate( bo, &(mi2->te[j]), (bo->next == 0));
      tree2 = newTREE();
      tree2->te = mi2->te[j];
      addTREEChild( tree1, tree2 );

      if ( mate( bo, tree2, gn + 1 ) == NOT_CHECK_MATE ) {
	/* ͤޤʤä */
	/* β(mi1->te[i])Ǥϵͤޤʤ */
	boBack_mate( bo, bo->next );
#ifdef USE_HASH_NO
	hsPutBOARD( bo, NOT_CHECK_MATE, bo->mi.count - gn, 0 );
#endif /* USE_HASH */
	goto NEXT;
      } else {
	count++;
      }
      boBack_mate( bo, bo->next );
    }
    if ( mi2->count == count ) {
      /* ͤ */

      freeMOVEINFO( mi1 );
      freeMOVEINFO( mi2 );

      boBack_mate( bo, bo->next );

#ifdef USE_HASH    
      hsPutBOARD( bo, CHECK_MATE, bo->mi.count - gn, 0 );
#endif /* USE_HASH */
      return CHECK_MATE;
    }

    NEXT:;

    freeMOVEINFO( mi2 );
    removeTREEChild( tree1 );
    boBack_mate( bo, bo->next );
  }
  
  freeMOVEINFO( mi1 );

  return NOT_CHECK_MATE;
}

/**
 * ſȤä;ؿ
 * bo ζ̤˵ͤߤ뤫Ĵ٤롣
 * ͤߤȯ tree ˼򥻥åȤ롣
 * ( tree ʲΥǡ¤ freeTREE() ȤäƳɬפ롣
 * freeTREE(tree->child) Τ褦ˤ롣 )
 * gn ϸߤɤߤο򥻥åȤ롣̾ 0 򥻥åȤǤ褤
 * ( gn  mate() ȤƵƽФȤ +1 ƥåȤ )
 * ɤߤο MATE_DEEP_MAX ˻ꤹ롣
 * @param bo оݤBOARD
 * @param gn 
 * @return CHECK_MATE ξ硢ͤߤ롣NOT_CHECK_MATE ϵͤߤդ
 *         ʤä
 */
MATESTATUS cmate(BOARD *bo, TREE *tree, int cn) {
  MOVEINFO *mi1, *mi2;
  TREE *tree1, *tree2;
  int count;
  int i, j;
  int king_xy;
  /* extern MOVEINFO g_mi[]; */
#ifdef USE_HASH
  extern long count_hash_hit;
  int mate_stat;
#endif /* USE_HASH */
  
  /* printf("cn = %d\n", cn); */

#ifdef USE_GUI
  /* GUIΥڥǥ󥰤줿٥Ȥ */
  if ((cn % 2) == 0) {
    processing_pending();
  }
#endif /* USE_GUI */

#ifdef USE_HASH
  if ( hsGetBOARD( bo, &mate_stat, bo->mi.count, 0 ) ) {
    count_hash_hit++;
    /* ̤ϿƤ */
    if ( mate_stat == CHECK_MATE ) {
      /* ͤߤηФƤΤĴ٤ʤ褤 */
      return CHECK_MATE;
    } else if ( mate_stat == NOT_CHECK_MATE ) {
      /* Եͤ */
#ifdef DEBUG
      assert(1); /* ߤԵͤߤϿƤʤ */
#endif /* DEBUG */
      return NOT_CHECK_MATE;
    } else {
      /*  */
    }
  }
#endif /* USE_HASH */

  mi1 = newMOVEINFO();
  /* mi1 = &(g_mi[ (gn << 1) + 1 ]); */
  
  ohte(bo, mi1);
  
#ifdef USE_HASH_NO
  if ( mi1->count == 0 ) {
    /* ꤬ʤ */
    hsPutBOARD( bo, NOT_CHECK_MATE, bo->mi.count, 0);
  }
#endif /* USE_HASH */

  for (i=0; i<mi1->count; i++) {
    boMove_mate( bo, &(mi1->te[i]), bo->next );
    king_xy = bo->code_xy[bo->next + 1][0];
    if (king_xy != 0 && boGetToBoardCount(king_xy, (bo->next == 0))) {
      /* ղˤʤäƤ */
      boBack_mate( bo, bo->next );
      continue;
    }

    mi2 = newMOVEINFO();
    /* mi2 = &(g_mi[(gn << 1) + 2]); */

    uke(bo, mi2, (bo->next == 0));
    
    if ( mi2->count == 0 && mi1->te[i].uti == FU ) {
      /* Ǥͤᡣ */
      freeMOVEINFO( mi2 );
      boBack_mate( bo, bo->next );
      continue;
    }
    
    tree1 = newTREE();
    tree1->te = mi1->te[i];
    addTREEChild( tree, tree1 );
    
    if ( mi2->count == 0 ) {
      /* ͤ */

      freeMOVEINFO( mi1 );
      freeMOVEINFO( mi2 );

      boBack_mate( bo, bo->next == 0);

#ifdef USE_HASH
      hsPutBOARD( bo, CHECK_MATE, bo->mi.count, 0);
#endif /* USE_HASH */
      return CHECK_MATE;
    }
    
    cn -= mi2->count;

    if (cn <= 0) {
      /* Եͤ */
      cn += mi2->count;
      
      freeMOVEINFO( mi2 );
      boBack_mate( bo, bo->next );
      removeTREEChild( tree1 );
      
      continue;
    }

    count = 0;
    for (j=0; j<mi2->count; j++) {
      boMove_mate( bo, &(mi2->te[j]), (bo->next == 0));
      tree2 = newTREE();
      tree2->te = mi2->te[j];
      addTREEChild( tree1, tree2 );

      if ( cmate( bo, tree2, cn) == NOT_CHECK_MATE ) {
	/* ͤޤʤä */
	/* β(mi1->te[i])Ǥϵͤޤʤ */
	boBack_mate( bo, bo->next );
#ifdef USE_HASH_NO
	hsPutBOARD( bo, NOT_CHECK_MATE, bo->mi.count - gn, 0 );
#endif /* USE_HASH */
	goto NEXT;
      } else {
	/* ͤ */
	count++;
      }
      boBack_mate( bo, bo->next );
    }
    if ( mi2->count == count ) {
      /* ͤ */

      freeMOVEINFO( mi1 );
      freeMOVEINFO( mi2 );

      boBack_mate( bo, bo->next );

#ifdef USE_HASH    
      hsPutBOARD( bo, CHECK_MATE, bo->mi.count, 0 );
#endif /* USE_HASH */
      return CHECK_MATE;
    }

    NEXT:;

    cn += mi2->count;
    
    freeMOVEINFO( mi2 );
    removeTREEChild( tree1 );
    boBack_mate( bo, bo->next );
  }
  
  freeMOVEINFO( mi1 );

  return NOT_CHECK_MATE;
}

/**
 * parent ˻Ҥ̤Υݥ child ä
 * @param parent ƤΥݥ
 * @param child ҤΥݥ
 */
void addTREEChild(TREE *parent, TREE *child) {
#ifdef DEBUG
  assert( parent != NULL );
  assert( child != NULL );
#endif /* DEBUG */

  if ( parent->child == NULL ) {
    parent->child = child;
    parent->child->parent = parent;
    parent->child->child = NULL;
    parent->child->brother = NULL;
  } else {
    TREE *tmp;
    tmp = parent->child;
    parent->child = child;
    parent->child->parent = parent;
    parent->child->child = NULL;
    parent->child->brother = tmp;
  }
}

/**
 * tree ηΥĥ꡼򤿤ɤä target ΰľηõƤ
 * ֤
 * @param tree оݤΥĥ꡼ؤΥݥ
 * @param target Υĥ꡼ؤΥݥ
 */
TREE *findPointerMe(TREE *tree, TREE *target) {
  TREE *tmp;
#ifdef DEBUG
  assert ( tree != NULL );
  assert ( target != NULL );
#endif /* DEBUG */

  tmp = tree;
  
  while ( 1 ) {
    if ( tmp->brother == target )
      break;
    tmp = tmp->brother;
#ifdef DEBUG
    assert ( tmp != NULL );
#endif /* DEBUG */
  }
  
  return tmp;
}

/**
 * Եͤꤷĥ꡼롣
 * @param tree ĥ꡼
 */
void removeTREEChild(TREE *tree) {
#ifdef DEBUG
  assert ( tree != NULL );
  assert ( tree->parent != NULL );
#endif /* DEBUG */

  if ( tree->child != NULL )
    freeTREE( tree->child );
  
  if ( tree->brother != NULL ) {
    if ( tree->parent->child == tree ) {
      tree->parent->child = tree->brother;
    } else {
      TREE *tmp;
      tmp = findPointerMe( tree->parent->child, tree );
      tmp->brother = tree->brother;
    }
  }
  
  tree->parent->child = NULL;

  free(tree);
}
